import { EditorView, ViewUpdate, BlockInfo } from "../../view/src";
import { Range, RangeValue, RangeSet } from "../../rangeset/src/rangeset";
import { ChangeSet } from "../../state/src";
export declare abstract class GutterMarker<T = any> extends RangeValue {
    readonly value: T;
    constructor(value: T);
    eq(other: GutterMarker<T>): boolean;
    map(mapping: ChangeSet, pos: number): Range<GutterMarker<T>> | null;
    toDOM(): Node | null;
    elementClass: string | null;
    static create<T>(pos: number, value: T): Range<GutterMarker<T>>;
    static set(of: Range<GutterMarker> | ReadonlyArray<Range<GutterMarker>>): GutterMarkerSet;
}
export declare type GutterMarkerSet = RangeSet<GutterMarker>;
export interface GutterConfig {
    class: string;
    fixed?: boolean;
    renderEmptyElements?: boolean;
    initialMarkers?: (view: EditorView) => GutterMarkerSet;
    updateMarkers?: (markers: GutterMarkerSet, update: ViewUpdate) => GutterMarkerSet;
    lineMarker?: (view: EditorView, line: BlockInfo, markers: ReadonlyArray<GutterMarker>) => GutterMarker | null;
    initialSpacer?: null | ((view: EditorView) => GutterMarker);
    updateSpacer?: null | ((spacer: GutterMarker, update: ViewUpdate) => GutterMarker);
}
export declare function gutter<T>(config: GutterConfig): import("../../extension/src/extension").Extension;
export interface LineNumberConfig {
    fixed?: boolean;
    formatNumber?: (lineNo: number) => string;
}
export declare type LineNumberMarkerUpdate = {
    add?: Range<GutterMarker>[];
    filter?: (from: number, to: number, marker: GutterMarker) => boolean;
};
export declare const lineNumberMarkers: import("../../extension/src/extension").SlotType<LineNumberMarkerUpdate>;
export declare const lineNumbers: (spec?: LineNumberConfig) => import("../../extension/src/extension").Extension;
