import { Text } from "../../doc/src";
import { EditorSelection } from "./selection";
import { Transaction } from "./transaction";
import { Extension, BehaviorStore } from "../../extension/src/extension";
export declare class StateExtension extends Extension {
    static allowMultipleSelections: import("../../extension/src/extension").Behavior<boolean>;
    static indentation: import("../../extension/src/extension").Behavior<(state: EditorState, pos: number) => number>;
}
declare class Configuration {
    readonly behavior: BehaviorStore;
    readonly fields: ReadonlyArray<StateField<any>>;
    readonly multipleSelections: boolean;
    readonly tabSize: number;
    readonly lineSeparator: string | null;
    constructor(behavior: BehaviorStore, fields: ReadonlyArray<StateField<any>>, multipleSelections: boolean, tabSize: number, lineSeparator: string | null);
    static create(config: EditorStateConfig): Configuration;
    updateTabSize(tabSize: number): Configuration;
    updateLineSeparator(lineSep: string | null): Configuration;
}
export interface EditorStateConfig {
    doc?: string | Text;
    selection?: EditorSelection;
    extensions?: ReadonlyArray<Extension>;
    tabSize?: number;
    lineSeparator?: string | null;
}
export declare class EditorState {
    readonly config: Configuration;
    private readonly fields;
    readonly doc: Text;
    readonly selection: EditorSelection;
    /** @internal */
    constructor(/* @internal */ config: Configuration, fields: ReadonlyArray<any>, doc: Text, selection: EditorSelection);
    getField<T>(field: StateField<T>): T;
    /** @internal */
    applyTransaction(tr: Transaction): EditorState;
    t(time?: number): Transaction;
    readonly tabSize: number;
    readonly multipleSelections: boolean;
    joinLines(text: ReadonlyArray<string>): string;
    splitLines(text: string): string[];
    readonly behavior: BehaviorStore;
    toJSON(): any;
    static fromJSON(json: any, config?: EditorStateConfig): EditorState;
    static create(config?: EditorStateConfig): EditorState;
}
export declare class StateField<T> {
    readonly init: (state: EditorState) => T;
    readonly apply: (tr: Transaction, value: T, newState: EditorState) => T;
    readonly extension: StateExtension;
    constructor({ init, apply }: {
        init: (state: EditorState) => T;
        apply: (tr: Transaction, value: T, newState: EditorState) => T;
    });
}
export {};
